@@ -75,8 +75,8 @@ struct TrackerMinerFilesPrivate {
 
 #if defined(HAVE_DEVKIT_POWER) || defined(HAVE_HAL)
 	TrackerPower *power;
-	gulong finished_handler;
 #endif /* defined(HAVE_DEVKIT_POWER) || defined(HAVE_HAL) */
+	gulong finished_handler;
 
 	DBusGProxy *extractor_proxy;
 
@@ -139,9 +139,10 @@ static void        index_recursive_directories_cb       (GObject              *g
 static void        index_single_directories_cb          (GObject              *gobject,
                                                          GParamSpec           *arg1,
                                                          gpointer              user_data);
-static void        ignore_directories_cb                (GObject              *gobject,
-							 GParamSpec           *arg1,
-							 gpointer              user_data);
+static gboolean    miner_files_force_recheck_idle       (gpointer user_data);
+static void        trigger_recheck_cb                   (GObject              *gobject,
+                                                         GParamSpec           *arg1,
+                                                         gpointer              user_data);
 static DBusGProxy *extractor_create_proxy               (void);
 static gboolean    miner_files_check_file               (TrackerMinerFS       *fs,
                                                          GFile                *file);
@@ -230,13 +231,12 @@ tracker_miner_files_init (TrackerMinerFiles *mf)
 	g_signal_connect (priv->power, "notify::on-battery",
 	                  G_CALLBACK (battery_status_cb),
 	                  mf);
+#endif /* defined(HAVE_DEVKIT_POWER) || defined(HAVE_HAL) */
 
 	priv->finished_handler = g_signal_connect_after (mf, "finished",
 							 G_CALLBACK (miner_finished_cb),
 							 NULL);
 
-#endif /* defined(HAVE_DEVKIT_POWER) || defined(HAVE_HAL) */
-
 	priv->volume_monitor = g_volume_monitor_get ();
 	g_signal_connect (priv->volume_monitor, "mount-pre-unmount",
 	                  G_CALLBACK (mount_pre_unmount_cb),
@@ -515,10 +515,13 @@ miner_files_constructed (GObject *object)
 	                  G_CALLBACK (index_single_directories_cb),
 	                  mf);
 	g_signal_connect (mf->private->config, "notify::ignored-directories",
-	                  G_CALLBACK (ignore_directories_cb),
+	                  G_CALLBACK (trigger_recheck_cb),
 	                  mf);
 	g_signal_connect (mf->private->config, "notify::ignored-directories-with-content",
-	                  G_CALLBACK (ignore_directories_cb),
+	                  G_CALLBACK (trigger_recheck_cb),
+	                  mf);
+	g_signal_connect (mf->private->config, "notify::ignored-files",
+	                  G_CALLBACK (trigger_recheck_cb),
 	                  mf);
 
 #if defined(HAVE_DEVKIT_POWER) || defined(HAVE_HAL)
@@ -544,15 +547,16 @@ set_up_mount_point_cb (GObject      *source,
                        gpointer      user_data)
 {
 	gchar *removable_device_urn = user_data;
-
 	GError *error = NULL;
+
 	tracker_miner_execute_update_finish (TRACKER_MINER (source),
 	                                     result,
 	                                     &error);
 
 	if (error) {
-		g_critical ("Could not set up mount point '%s': %s",
-		            removable_device_urn, error->message);
+		g_critical ("Could not set mount point in database '%s', %s",
+		            removable_device_urn,
+			    error->message);
 		g_error_free (error);
 	}
 
@@ -568,8 +572,7 @@ set_up_mount_point (TrackerMinerFiles *miner,
 {
 	GString *queries;
 
-	g_debug ("Setting mount point '%s' state in database (URN '%s')",
-	         mount_point,
+	g_debug ("Mount point state being set in DB for URN '%s'",
 	         removable_device_urn);
 
 	queries = g_string_new (NULL);
@@ -706,27 +709,49 @@ query_mount_points_cb (GObject      *source,
 	                                 (GDestroyNotify) g_free,
 	                                 NULL);
 
+	/* Make sure the root partition is always set to mounted, as GIO won't
+	 * report it as a proper mount */
+	g_hash_table_insert (volumes,
+	                     g_strdup (TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN),
+	                     GINT_TO_POINTER (VOLUME_MOUNTED));
+
+	/* Get mounted status from store */
 	for (i = 0; i < query_results->len; i++) {
 		gchar **row;
-		gint state;
 
 		row = g_ptr_array_index (query_results, i);
-		state = VOLUME_MOUNTED_IN_STORE;
 
 		if (strcmp (row[0], TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN) == 0) {
-			/* Report non-removable media to be mounted by HAL as well */
-			state |= VOLUME_MOUNTED;
+			/* Update root partition to set also mounted in store */
+			g_hash_table_replace (volumes,
+			                     g_strdup (TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN),
+			                     GINT_TO_POINTER (VOLUME_MOUNTED | VOLUME_MOUNTED_IN_STORE));
+		} else {
+			/* Set status of known volumes in store */
+			g_hash_table_insert (volumes,
+			                     g_strdup (row[0]),
+			                     GINT_TO_POINTER (VOLUME_MOUNTED_IN_STORE));
 		}
-
-		g_hash_table_insert (volumes, g_strdup (row[0]), GINT_TO_POINTER (state));
 	}
 
-	g_hash_table_replace (volumes, g_strdup (TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN),
-	                      GINT_TO_POINTER (VOLUME_MOUNTED));
+	/* Then, get all currently mounted non-REMOVABLE volumes, according to GIO */
+	uuids = tracker_storage_get_device_uuids (priv->storage, 0, TRUE);
+	for (u = uuids; u; u = u->next) {
+		const gchar *uuid;
+		gchar *non_removable_device_urn;
+		gint state;
 
+		uuid = u->data;
+		non_removable_device_urn = g_strdup_printf (TRACKER_DATASOURCE_URN_PREFIX "%s", uuid);
+
+		state = GPOINTER_TO_INT (g_hash_table_lookup (volumes, non_removable_device_urn));
+		state |= VOLUME_MOUNTED;
+
+		g_hash_table_replace (volumes, non_removable_device_urn, GINT_TO_POINTER (state));
+	}
+
+	/* Then, get all currently mounted REMOVABLE volumes, according to GIO */
 	uuids = tracker_storage_get_device_uuids (priv->storage, TRACKER_STORAGE_REMOVABLE, FALSE);
-
-	/* Then, get all currently mounted volumes, according to GIO */
 	for (u = uuids; u; u = u->next) {
 		const gchar *uuid;
 		gchar *removable_device_urn;
@@ -765,18 +790,33 @@ query_mount_points_cb (GObject      *source,
 
 			if (urn) {
 				if (mount_point) {
-					g_debug ("URN '%s' (mount point: %s) was not reported to be mounted, but now it is, updating state",
-					         urn, mount_point);
+					g_debug ("Mount point state incorrect in DB for URN '%s', "
+						 "currently it is mounted on '%s'",
+					         urn,
+						 mount_point);
 				} else {
-					g_debug ("URN '%s' was not reported to be mounted, but now it is, updating state", urn);
+					g_debug ("Mount point state incorrect in DB for URN '%s', "
+						 "currently it is mounted",
+					         urn);
 				}
-				set_up_mount_point (TRACKER_MINER_FILES (miner), urn, mount_point, TRUE, accumulator);
+
+				set_up_mount_point (TRACKER_MINER_FILES (miner),
+						    urn,
+						    mount_point,
+						    TRUE,
+						    accumulator);
 			}
 		} else if (!(state & VOLUME_MOUNTED) &&
 		           (state & VOLUME_MOUNTED_IN_STORE)) {
 			if (urn) {
-				g_debug ("URN '%s' was reported to be mounted, but it isn't anymore, updating state", urn);
-				set_up_mount_point (TRACKER_MINER_FILES (miner), urn, NULL, FALSE, accumulator);
+				g_debug ("Mount pont state incorrect in DB for URN '%s', "
+					 "currently it is NOT mounted",
+					 urn);
+				set_up_mount_point (TRACKER_MINER_FILES (miner),
+						    urn,
+						    NULL,
+						    FALSE,
+						    accumulator);
 			}
 		}
 	}
@@ -814,13 +854,22 @@ mount_point_removed_cb (TrackerStorage *storage,
 {
 	TrackerMinerFiles *miner = user_data;
 	gchar *urn;
-
-	g_debug ("Removing mount point '%s'", mount_point);
+	GFile *mount_point_file;
 
 	urn = g_strdup_printf (TRACKER_DATASOURCE_URN_PREFIX "%s", uuid);
+	g_debug ("Mount point removed for URN '%s'", urn);
 
+	mount_point_file = g_file_new_for_path (mount_point);
+
+	/* Set mount point status in tracker-store */
 	set_up_mount_point (miner, urn, mount_point, FALSE, NULL);
+
+	/* Tell TrackerMinerFS to skip monitoring everything under the mount
+	 *  point (in case there was no pre-unmount notification) */
+	tracker_miner_fs_directory_remove (TRACKER_MINER_FS (miner), mount_point_file);
+
 	g_free (urn);
+	g_object_unref (mount_point_file);
 }
 
 static void
@@ -833,33 +882,84 @@ mount_point_added_cb (TrackerStorage *storage,
 {
 	TrackerMinerFiles *miner = user_data;
 	TrackerMinerFilesPrivate *priv;
-	GFile *file;
 	gchar *urn;
 	gboolean index_removable_devices;
 	gboolean index_optical_discs;
-	gboolean should_crawl;
 
 	priv = TRACKER_MINER_FILES_GET_PRIVATE (miner);
 
 	index_removable_devices = tracker_config_get_index_removable_devices (priv->config);
 	index_optical_discs = tracker_config_get_index_optical_discs (priv->config);
 
-	g_message ("Added mount point '%s'", mount_point);
+	urn = g_strdup_printf (TRACKER_DATASOURCE_URN_PREFIX "%s", uuid);
+	g_message ("Mount point added for URN '%s'", urn);
 
-	should_crawl = TRUE;
-
-	if (removable && !tracker_config_get_index_removable_devices (priv->config)) {
+	if (removable && !index_removable_devices) {
 		g_message ("  Not crawling, removable devices disabled in config");
-		should_crawl = FALSE;
-	}
-
-	if (optical && !tracker_config_get_index_optical_discs (priv->config)) {
+	} else if (optical && !index_optical_discs) {
 		g_message ("  Not crawling, optical devices discs disabled in config");
-		should_crawl = FALSE;
-	}
+	} else if (!removable && !optical) {
+		GFile *mount_point_file;
+		GSList *l;
 
-	if (should_crawl) {
-		g_message ("  Adding directory to crawler's queue");
+		mount_point_file = g_file_new_for_path (mount_point);
+
+		/* Check if one of the recursively indexed locations is in
+		 *   the mounted path, or if the mounted path is inside
+		 *   a recursively indexed directory... */
+		for (l = tracker_config_get_index_recursive_directories (miner->private->config);
+		     l;
+		     l = g_slist_next (l)) {
+			GFile *config_file;
+
+			config_file = g_file_new_for_path (l->data);
+
+			if (g_file_equal (config_file, mount_point_file) ||
+			    g_file_has_prefix (config_file, mount_point_file)) {
+				/* If the config path is contained inside the mount path,
+				 *  then add the config path to re-check */
+				g_message ("  Re-check of configured path '%s' needed (recursively)",
+				           (gchar *) l->data);
+				tracker_miner_fs_directory_add (TRACKER_MINER_FS (user_data),
+				                                config_file,
+				                                TRUE);
+			} else if (g_file_has_prefix (mount_point_file, config_file)) {
+				/* If the mount path is contained inside the config path,
+				 *  then add the mount path to re-check */
+				g_message ("  Re-check of path '%s' needed (inside configured path '%s')",
+				           mount_point,
+				           (gchar *) l->data);
+				tracker_miner_fs_directory_add (TRACKER_MINER_FS (user_data),
+				                                mount_point_file,
+				                                TRUE);
+			}
+			g_object_unref (config_file);
+		}
+
+		/* Check if one of the non-recursively indexed locations is in
+		 *  the mount path... */
+		for (l = tracker_config_get_index_single_directories (miner->private->config);
+		     l;
+		     l = g_slist_next (l)) {
+			GFile *config_file;
+
+			config_file = g_file_new_for_path (l->data);
+			if (g_file_equal (config_file, mount_point_file) ||
+			    g_file_has_prefix (config_file, mount_point_file)) {
+				g_message ("  Re-check of configured path '%s' needed (non-recursively)",
+				           (gchar *) l->data);
+				tracker_miner_fs_directory_add (TRACKER_MINER_FS (user_data),
+				                                config_file,
+				                                FALSE);
+			}
+			g_object_unref (config_file);
+		}
+
+		g_object_unref (mount_point_file);
+	} else {
+		GFile *file;
+
+		g_message ("  Adding directories in removable/optical media to crawler's queue");
 
 		file = g_file_new_for_path (mount_point);
 		g_object_set_qdata_full (G_OBJECT (file),
@@ -877,7 +977,6 @@ mount_point_added_cb (TrackerStorage *storage,
 		g_object_unref (file);
 	}
 
-	urn = g_strdup_printf (TRACKER_DATASOURCE_URN_PREFIX "%s", uuid);
 	set_up_mount_point (miner, urn, mount_point, TRUE, NULL);
 	g_free (urn);
 }
@@ -986,6 +1085,8 @@ index_on_battery_cb (GObject    *object,
 	check_battery_status (mf);
 }
 
+#endif /* defined(HAVE_DEVKIT_POWER) || defined(HAVE_HAL) */
+
 /* Called when mining has finished the first time */
 static void
 miner_finished_cb (TrackerMinerFS *fs,
@@ -1010,10 +1111,10 @@ miner_finished_cb (TrackerMinerFS *fs,
 		mf->private->finished_handler = 0;
 	}
 
+#if defined(HAVE_DEVKIT_POWER) || defined(HAVE_HAL)
 	check_battery_status (mf);
-}
-
 #endif /* defined(HAVE_DEVKIT_POWER) || defined(HAVE_HAL) */
+}
 
 static void
 mount_pre_unmount_cb (GVolumeMonitor    *volume_monitor,
@@ -1021,10 +1122,14 @@ mount_pre_unmount_cb (GVolumeMonitor    *volume_monitor,
                       TrackerMinerFiles *mf)
 {
 	GFile *mount_root;
+	gchar *uri;
 
 	mount_root = g_mount_get_root (mount);
+	uri = g_file_get_uri (mount_root);
+	g_message ("Pre-unmount requested for '%s'", uri);
 	tracker_miner_fs_directory_remove (TRACKER_MINER_FS (mf), mount_root);
 	g_object_unref (mount_root);
+	g_free (uri);
 }
 
 static gboolean
@@ -1257,9 +1362,9 @@ miner_files_force_recheck_idle (gpointer user_data)
 }
 
 static void
-ignore_directories_cb (GObject    *gobject,
-                       GParamSpec *arg1,
-                       gpointer    user_data)
+trigger_recheck_cb (GObject    *gobject,
+                    GParamSpec *arg1,
+                    gpointer    user_data)
 {
 	TrackerMinerFiles *miner_files = user_data;
 
