@@ -70,9 +70,8 @@
 
 #include <libedataserver/e-account.h>
 #include <libedataserver/e-account-list.h>
-#include <libedataserver/eds-version.h>
 
-#if EDS_CHECK_VERSION (2, 29, 1)
+#ifdef HAVE_EDS_2_29_1
 #include <e-util/e-account-utils.h>
 #endif
 
@@ -133,6 +132,8 @@ typedef struct {
 	GMutex *mutex;
 	GCond *cond;
 	gboolean has_happened;
+	gpointer pool;
+	gboolean dont_free;
 } PoolItem;
 
 typedef struct {
@@ -340,6 +341,7 @@ static void
 thread_pool_exec (gpointer data, gpointer user_data)
 {
 	ThreadPool *pool = user_data;
+	PoolItem *item;
 	gboolean dying;
 
 	g_mutex_lock (pool->mutex);
@@ -347,10 +349,13 @@ thread_pool_exec (gpointer data, gpointer user_data)
 	pool->items = g_list_remove (pool->items, data);
 	g_mutex_unlock (pool->mutex);
 
+
 	if (!dying)
 		pool->func (data, pool->cancel);
 
-	pool->freeup (data, pool->cancel);
+	item = data;
+	if (!item->dont_free)
+		pool->freeup (data, pool->cancel);
 }
 
 
@@ -358,6 +363,7 @@ static void
 reply_void (GError *error, gpointer  user_data)
 {
 	PoolItem *item = user_data;
+	ThreadPool *pool = item->pool;
 
 	if (error) {
 		g_debug ("Tracker plugin: Error updating data: %s\n", error->message);
@@ -367,6 +373,9 @@ reply_void (GError *error, gpointer  user_data)
 	g_cond_broadcast (item->cond);
 	item->has_happened = TRUE;
 	g_mutex_unlock (item->mutex);
+
+	if (item->dont_free)
+		pool->freeup (item, pool->cancel);
 }
 
 static void
@@ -396,6 +405,7 @@ exec_update (gpointer data, gpointer user_data)
 		g_get_current_time (&val);
 		g_time_val_add (&val, 5 * 1000000); /* 5 seconds worth of patience */
 		no_patience = g_cond_timed_wait (item->cond, item->mutex, &val);
+		item->dont_free = !no_patience;
 	}
 	g_mutex_unlock (item->mutex);
 
@@ -438,9 +448,11 @@ thread_pool_new (GFunc func, GFunc freeup, GCompareDataFunc sorter)
 static void
 thread_pool_push (ThreadPool *pool, gpointer item, gpointer user_data)
 {
+	g_mutex_lock (pool->mutex);
 	pool->items = g_list_prepend (pool->items, item);
 	if (!pool->dying)
 		g_thread_pool_push (pool->pool, item, user_data);
+	g_mutex_unlock (pool->mutex);
 }
 
 static gpointer
@@ -481,6 +493,8 @@ send_sparql_update (TrackerEvolutionPlugin *self, const gchar *sparql, gint prio
 		if (!sparql_pool)
 			sparql_pool = thread_pool_new (exec_update, free_pool_item, pool_sort_func);
 
+		item->pool = sparql_pool;
+		item->dont_free = FALSE;
 		item->prio = prio;
 		item->commit = FALSE;
 		item->client = g_object_ref (priv->client);
@@ -514,6 +528,8 @@ send_sparql_commit (TrackerEvolutionPlugin *self, gboolean update)
 		if (!sparql_pool)
 			sparql_pool = thread_pool_new (exec_update, free_pool_item, pool_sort_func);
 
+		item->pool = sparql_pool;
+		item->dont_free = FALSE;
 		item->prio = 0;
 		item->commit = TRUE;
 		item->client = g_object_ref (priv->client);
@@ -748,6 +764,7 @@ on_folder_summary_changed (CamelFolder *folder,
 
 	summary = folder->summary;
 	em_uri = em_uri_from_camel (account_uri);
+	em_uri [strlen (em_uri) - 1] = '\0';
 
 	merged = g_ptr_array_new ();
 
@@ -814,10 +831,10 @@ on_folder_summary_changed (CamelFolder *folder,
 			/* This is not a path but a URI, don't use the
 			 * OS's directory separator here */
 
-			uri = g_strdup_printf ("%s%s/%s",
-			                       em_uri,
-			                       camel_folder_get_full_name (folder),
-			                       uid);
+			uri = tracker_uri_printf_escaped ("%s/%s/%s",
+			                                  em_uri,
+			                                  camel_folder_get_full_name (folder),
+			                                  uid);
 
 			sparql = tracker_sparql_builder_new_update ();
 
@@ -895,17 +912,17 @@ on_folder_summary_changed (CamelFolder *folder,
 		GString *sparql = g_string_new ("");
 
 		for (i = 0; i< changes->uid_removed->len; i++) {
+			gchar *uri;
 
 			/* This is not a path but a URI, don't use the OS's
 			 * directory separator here */
+			uri = tracker_uri_printf_escaped ("%s/%s/%s",
+			                                  em_uri,
+			                                  camel_folder_get_full_name (folder),
+			                                  (char*) changes->uid_removed->pdata[i]);
 
-			g_string_append_printf (sparql, "DELETE FROM <%s%s/%s> { <%s%s/%s> a rdfs:Resource }\n ",
-			                        em_uri,
-			                        camel_folder_get_full_name (folder),
-			                        (char*) changes->uid_removed->pdata[i],
-			                        em_uri,
-			                        camel_folder_get_full_name (folder),
-			                        (char*) changes->uid_removed->pdata[i]);
+			g_string_append_printf (sparql, "DELETE FROM <%s> { <%s> a rdfs:Resource }\n ", uri, uri);
+			g_free (uri);
 		}
 
 		send_sparql_update (info->self, sparql->str, 100);
@@ -938,6 +955,7 @@ introduce_walk_folders_in_folder (TrackerEvolutionPlugin *self,
 	}
 
 	em_uri = em_uri_from_camel (account_uri);
+	em_uri [strlen (em_uri) - 1] = '\0';
 
 	while (iter) {
 		guint count = 0;
@@ -1018,8 +1036,7 @@ introduce_walk_folders_in_folder (TrackerEvolutionPlugin *self,
 
 				folder = g_hash_table_lookup (priv->cached_folders, iter->full_name);
 
-				uri =  g_strdup_printf ("%s%s/%s", em_uri,
-				                        iter->full_name, uid);
+				uri = tracker_uri_printf_escaped ("%s/%s/%s", em_uri, iter->full_name, uid);
 
 				if (!sparql) {
 					sparql = tracker_sparql_builder_new_update ();
@@ -1142,7 +1159,10 @@ introduce_store_deal_with_deleted (TrackerEvolutionPlugin *self,
 	sqlite3_stmt *stmt = NULL;
 	CamelDB *cdb_r;
 	guint i, ret;
-	gchar *em_uri = em_uri_from_camel (account_uri);
+	gchar *em_uri;
+
+	em_uri = em_uri_from_camel (account_uri);
+	em_uri [strlen (em_uri) - 1] = '\0';
 
 	query = sqlite3_mprintf ("SELECT uid, mailbox "
 	                         "FROM Deletes "
@@ -1184,8 +1204,9 @@ introduce_store_deal_with_deleted (TrackerEvolutionPlugin *self,
 			/* This is not a path but a URI, don't use the OS's
 			 * directory separator here */
 
-			g_ptr_array_add (subjects_a, g_strdup_printf ("%s%s/%s", em_uri,
-			                                              mailbox, uid));
+			g_ptr_array_add (subjects_a,
+			                 tracker_uri_printf_escaped ("%s/%s/%s", em_uri,
+			                                             mailbox, uid));
 
 			if (count > 100) {
 				more = TRUE;
@@ -2289,7 +2310,7 @@ tracker_evolution_plugin_init (TrackerEvolutionPlugin *plugin)
 	                         G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
 	                         G_TYPE_INVALID);
 
-#if EDS_CHECK_VERSION (2, 29, 1)
+#ifdef HAVE_EDS_2_29_1
 	priv->accounts = g_object_ref (e_get_account_list ());
 #else
 	priv->accounts = g_object_ref (mail_config_get_accounts ());
@@ -2345,7 +2366,7 @@ miner_started (TrackerMiner *miner)
 	                         G_TYPE_INVALID,
 	                         G_TYPE_INVALID);
 
-	g_object_set (miner,  "progress", 0.0,  "status", _("Initializing"), NULL);
+	g_object_set (miner,  "progress", 0.0,  "status", "Initializing", NULL);
 }
 
 static void
@@ -2437,7 +2458,7 @@ miner_resumed (TrackerMiner *miner)
 		priv->client = tracker_client_new (0, G_MAXINT);
 	}
 
-	g_object_set (miner,  "progress", 0.0,  "status", _("Resuming"), NULL);
+	g_object_set (miner,  "progress", 0.0,  "status", _("Processing…"), NULL);
 
 	dbus_g_proxy_begin_call (priv->dbus_proxy, "ListNames",
 	                         list_names_reply_cb,
